"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Processor = void 0;
const ymir_1 = require("@fimbul/ymir");
const path = require("path");
const SAXParser = require("parse5-sax-parser");
const voidElements = require("void-elements");
class Processor extends ymir_1.AbstractProcessor {
    constructor(context) {
        super(context);
        this.range = {
            start: 0,
            end: Infinity,
            line: 0,
            firstLineOffset: 0,
        };
        if (path.extname(this.sourceFileName) === '.vue') {
            const parser = new SAXParser({ sourceCodeLocationInfo: true });
            let depth = 0;
            parser.on('startTag', (startTag) => {
                if (startTag.selfClosing || voidElements[startTag.tagName])
                    return;
                ++depth;
                if (depth === 1 && startTag.tagName === 'script')
                    this.range.start = startTag.sourceCodeLocation.endOffset;
            });
            parser.on('endTag', (endTag) => {
                --depth;
                if (depth === 0 && endTag.tagName === 'script')
                    this.range.end = endTag.sourceCodeLocation.startOffset;
            });
            parser.write(this.source);
            parser.end();
            const lineBreakAfterStart = /^\r?\n/.exec(this.source.substr(this.range.start));
            if (lineBreakAfterStart !== null) {
                this.range.start += lineBreakAfterStart[0].length;
            }
            else {
                let lineBreakBeforeStart = this.source.lastIndexOf('\n', this.range.start);
                if (lineBreakBeforeStart === -1)
                    lineBreakBeforeStart = 0;
                this.range.firstLineOffset = this.range.start - lineBreakBeforeStart;
            }
            const match = this.source.substring(0, this.range.start).match(/(\r?\n)/g);
            if (match !== null)
                this.range.line = match.length;
        }
    }
    static getSuffixForFile(context) {
        if (path.extname(context.fileName) !== '.vue')
            return '';
        let suffix = '';
        const parser = new SAXParser();
        let depth = 0;
        parser.on('startTag', (startTag) => {
            if (startTag.selfClosing || voidElements[startTag.tagName])
                return;
            ++depth;
            if (depth === 1 && startTag.tagName === 'script') {
                const lang = startTag.attrs.find((attr) => attr.name === 'lang');
                suffix = lang === undefined ? '.js' : `.${lang.value}`;
            }
        });
        parser.on('endTag', () => {
            --depth;
        });
        parser.write(context.readFile());
        parser.end();
        return suffix;
    }
    preprocess() {
        return this.source.substring(this.range.start, this.range.end);
    }
    updateSource(newSource, changeRange) {
        const diff = newSource.length - this.source.length;
        this.range.end += diff;
        this.source = newSource;
        return {
            transformed: this.preprocess(),
            changeRange: {
                span: {
                    start: changeRange.span.start - this.range.start,
                    length: changeRange.span.length,
                },
                newLength: changeRange.newLength,
            },
        };
    }
    postprocess(findings) {
        return findings.map(this.mapFinding, this);
    }
    mapFinding(f) {
        return {
            ...f,
            start: this.adjustPosition(f.start),
            end: this.adjustPosition(f.end),
            fix: f.fix === undefined
                ? undefined
                : {
                    replacements: f.fix.replacements.map(this.mapReplacement, this),
                },
        };
    }
    adjustPosition(pos) {
        return {
            // special handling for first line if there is no line break after opening tag
            character: pos.line === 0 ? pos.character + this.range.firstLineOffset : pos.character,
            line: pos.line + this.range.line,
            position: pos.position + this.range.start,
        };
    }
    mapReplacement(r) {
        return {
            start: r.start + this.range.start,
            end: r.end + this.range.start,
            text: r.text,
        };
    }
}
exports.Processor = Processor;
//# sourceMappingURL=index.js.map